@@ -39,8 +39,6 @@ type Stackframe struct {
 	stackHi uint64
 	// Return address for this stack frame (as read from the stack frame itself).
 	Ret uint64
-	// Address to the memory location containing the return address
-	addrret uint64
 	// Err is set if an error occurred during stacktrace
 	Err error
 	// SystemStack is true if this frame belongs to a system stack.
@@ -255,6 +253,9 @@ func (it *stackIterator) Err() error {
 // frameBase calculates the frame base pseudo-register for DWARF for fn and
 // the current frame.
 func (it *stackIterator) frameBase(fn *Function) int64 {
+	if fn.cu.image.Stripped() {
+		return 0
+	}
 	dwarfTree, err := fn.cu.image.getDwarfTree(fn.offset)
 	if err != nil {
 		return 0
@@ -275,7 +276,10 @@ func (it *stackIterator) newStackframe(ret, retaddr uint64) Stackframe {
 	} else {
 		it.regs.FrameBase = it.frameBase(fn)
 	}
-	r := Stackframe{Current: Location{PC: it.pc, File: f, Line: l, Fn: fn}, Regs: it.regs, Ret: ret, addrret: retaddr, stackHi: it.stackhi, SystemStack: it.systemstack, lastpc: it.pc}
+	r := Stackframe{Current: Location{PC: it.pc, File: f, Line: l, Fn: fn}, Regs: it.regs, Ret: ret, stackHi: it.stackhi, SystemStack: it.systemstack, lastpc: it.pc}
+	if r.Regs.Reg(it.regs.PCRegNum) == nil {
+		r.Regs.AddReg(it.regs.PCRegNum, op.DwarfRegisterFromUint64(it.pc))
+	}
 	r.Call = r.Current
 	if !it.top && r.Current.Fn != nil && it.pc != r.Current.Fn.Entry {
 		// if the return address is the entry point of the function that
@@ -363,7 +367,6 @@ func (it *stackIterator) appendInlineCalls(frames []Stackframe, frame Stackframe
 			Regs:        frame.Regs,
 			stackHi:     frame.stackHi,
 			Ret:         frame.Ret,
-			addrret:     frame.addrret,
 			Err:         frame.Err,
 			SystemStack: frame.SystemStack,
 			Inlined:     true,
@@ -397,14 +400,21 @@ func (it *stackIterator) advanceRegs() (callFrameRegs op.DwarfRegisters, ret uin
 
 	cfareg, err := it.executeFrameRegRule(0, framectx.CFA, 0)
 	if cfareg == nil {
-		it.err = fmt.Errorf("CFA becomes undefined at PC %#x", it.pc)
+		it.err = fmt.Errorf("CFA becomes undefined at PC %#x: %v", it.pc, err)
 		return op.DwarfRegisters{}, 0, 0
 	}
 	it.regs.CFA = int64(cfareg.Uint64Val)
 
 	callimage := it.bi.PCToImage(it.pc)
 
-	callFrameRegs = op.DwarfRegisters{StaticBase: callimage.StaticBase, ByteOrder: it.regs.ByteOrder, PCRegNum: it.regs.PCRegNum, SPRegNum: it.regs.SPRegNum, BPRegNum: it.regs.BPRegNum, LRRegNum: it.regs.LRRegNum}
+	callFrameRegs = op.DwarfRegisters{
+		StaticBase: callimage.StaticBase,
+		ByteOrder:  it.regs.ByteOrder,
+		PCRegNum:   it.regs.PCRegNum,
+		SPRegNum:   it.regs.SPRegNum,
+		BPRegNum:   it.regs.BPRegNum,
+		LRRegNum:   it.regs.LRRegNum,
+	}
 
 	// According to the standard the compiler should be responsible for emitting
 	// rules for the RSP register so that it can then be used to calculate CFA,
@@ -421,6 +431,7 @@ func (it *stackIterator) advanceRegs() (callFrameRegs op.DwarfRegisters, ret uin
 		if i == framectx.RetAddrReg {
 			if reg == nil {
 				if err == nil {
+					//lint:ignore ST1005 backwards compatibility
 					err = fmt.Errorf("Undefined return address at %#x", it.pc)
 				}
 				it.err = err
@@ -575,13 +586,14 @@ func (g *G) readDefers(frames []Stackframe) {
 }
 
 func (d *Defer) load() {
-	d.variable.loadValue(LoadConfig{false, 1, 0, 0, -1, 0})
-	if d.variable.Unreadable != nil {
-		d.Unreadable = d.variable.Unreadable
+	v := d.variable // +rtype _defer
+	v.loadValue(LoadConfig{false, 1, 0, 0, -1, 0})
+	if v.Unreadable != nil {
+		d.Unreadable = v.Unreadable
 		return
 	}
 
-	fnvar := d.variable.fieldVariable("fn")
+	fnvar := v.fieldVariable("fn")
 	if fnvar.Kind == reflect.Func {
 		// In Go 1.18, fn is a func().
 		d.DwrapPC = fnvar.Base
@@ -593,9 +605,9 @@ func (d *Defer) load() {
 		}
 	}
 
-	d.DeferPC, _ = constant.Uint64Val(d.variable.fieldVariable("pc").Value)
-	d.SP, _ = constant.Uint64Val(d.variable.fieldVariable("sp").Value)
-	sizVar := d.variable.fieldVariable("siz")
+	d.DeferPC, _ = constant.Uint64Val(v.fieldVariable("pc").Value) // +rtype uintptr
+	d.SP, _ = constant.Uint64Val(v.fieldVariable("sp").Value)      // +rtype uintptr
+	sizVar := v.fieldVariable("siz")                               // +rtype -opt int32
 	if sizVar != nil {
 		// In Go <1.18, siz stores the number of bytes of
 		// defer arguments following the defer record. In Go
@@ -604,14 +616,14 @@ func (d *Defer) load() {
 		d.argSz, _ = constant.Int64Val(sizVar.Value)
 	}
 
-	linkvar := d.variable.fieldVariable("link").maybeDereference()
+	linkvar := v.fieldVariable("link").maybeDereference() // +rtype *_defer
 	if linkvar.Addr != 0 {
 		d.link = &Defer{variable: linkvar}
 	}
 }
 
 // errSPDecreased is used when (*Defer).Next detects a corrupted linked
-// list, specifically when after followin a link pointer the value of SP
+// list, specifically when after following a link pointer the value of SP
 // decreases rather than increasing or staying the same (the defer list is a
 // FIFO list, nodes further down the list have been added by function calls
 // further down the call stack and therefore the SP should always increase).
@@ -686,6 +698,6 @@ func (d *Defer) DeferredFunc(p *Target) (file string, line int, fn *Function) {
 	if fn == nil {
 		return "", 0, nil
 	}
-	file, line = fn.cu.lineInfo.PCToLine(fn.Entry, fn.Entry)
+	file, line = bi.EntryLineForFunc(fn)
 	return file, line, fn
 }
